/* eslint-disable no-magic-numbers */
import { render, screen, waitFor } from '@testing-library/react';
import {
  InitialStoreState,
  getReduxWrapperWithStore,
} from '@/utils/testing/getReduxWrapperWithStore';
import { StoreType } from '@/redux/store';
import { act } from 'react-dom/test-utils';
import { compartmentPathwaysDetailsFixture } from '@/models/fixtures/compartmentPathways';
import { IncludedCompartmentPathways } from './IncludedCompartmentPathways.component';

const renderComponent = (initialStoreState: InitialStoreState = {}): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStoreState);

  return (
    render(
      <Wrapper>
        <IncludedCompartmentPathways />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

const CHECKBOX_ELEMENT_NAME = compartmentPathwaysDetailsFixture[0].name;

describe('IncludedCompartmentPathways - component', () => {
  it('should display compartment / pathways checkboxes when fetching data is successful', async () => {
    renderComponent({
      compartmentPathways: {
        data: compartmentPathwaysDetailsFixture,
        loading: 'succeeded',
        error: {
          message: '',
          name: '',
        },
      },
    });

    expect(screen.queryByTestId('checkbox-filter')).not.toBeVisible();

    const navigationButton = screen.getByTestId('accordion-item-button');

    act(() => {
      navigationButton.click();
    });

    expect(screen.getByText('Select included compartment / pathways')).toBeInTheDocument();

    await waitFor(() => {
      expect(screen.getByTestId('checkbox-filter')).toBeInTheDocument();
      expect(screen.getByLabelText('search-input')).toBeInTheDocument();
      expect(screen.getByLabelText(CHECKBOX_ELEMENT_NAME)).toBeInTheDocument();
    });
  });
  it('should not display compartment / pathways checkboxes when fetching data fails', async () => {
    renderComponent({
      compartmentPathways: {
        data: [],
        loading: 'failed',
        error: {
          message: '',
          name: '',
        },
      },
    });
    expect(screen.getByText('Select included compartment / pathways')).toBeInTheDocument();
    const navigationButton = screen.getByTestId('accordion-item-button');
    act(() => {
      navigationButton.click();
    });

    expect(screen.queryByTestId('checkbox-filter')).not.toBeInTheDocument();
  });
  it('should not display compartment / pathways checkboxes when fetched data is empty', async () => {
    renderComponent({
      compartmentPathways: {
        data: [],
        loading: 'succeeded',
        error: {
          message: '',
          name: '',
        },
      },
    });
    expect(screen.getByText('Select included compartment / pathways')).toBeInTheDocument();
    const navigationButton = screen.getByTestId('accordion-item-button');
    act(() => {
      navigationButton.click();
    });

    expect(screen.queryByTestId('checkbox-filter')).not.toBeInTheDocument();
  });

  it('should display loading message when fetching data is pending', async () => {
    renderComponent({
      compartmentPathways: {
        data: [],
        loading: 'pending',
        error: {
          message: '',
          name: '',
        },
      },
    });
    expect(screen.getByText('Select included compartment / pathways')).toBeInTheDocument();
    const navigationButton = screen.getByTestId('accordion-item-button');
    act(() => {
      navigationButton.click();
    });

    expect(screen.getByText('Loading...')).toBeInTheDocument();
  });
});
