/* eslint-disable no-magic-numbers */
import {
  ELEMENT_SEARCH_RESULT_MOCK_ALIAS,
  ELEMENT_SEARCH_RESULT_MOCK_REACTION,
} from '@/models/mocks/elementSearchResultMock';
import { apiPath } from '@/redux/apiPath';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import { getSearchResults } from './getSearchResults';

const mockedAxiosOldClient = mockNetworkResponse();

describe('getSearchResults - util', () => {
  describe('when results type is ALIAS', () => {
    const { modelId } = ELEMENT_SEARCH_RESULT_MOCK_ALIAS;
    const mapSize = {
      width: 270,
      height: 270,
      tileSize: 256,
      minZoom: 2,
      maxZoom: 9,
    };
    const coordinate = [270, 270];
    const point = { x: 540.0072763538013, y: 539.9927236461986 };

    beforeAll(() => {
      mockedAxiosOldClient
        .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
        .reply(HttpStatusCode.Ok, [ELEMENT_SEARCH_RESULT_MOCK_ALIAS]);
    });

    it('returns valid array of objects', async () => {
      const result = await getSearchResults({
        coordinate,
        mapSize,
        modelId,
      });

      expect(result).toEqual({
        point,
        searchResults: [ELEMENT_SEARCH_RESULT_MOCK_ALIAS],
      });
    });
  });

  describe('when results type is REACTION', () => {
    const { modelId } = ELEMENT_SEARCH_RESULT_MOCK_REACTION;
    const mapSize = {
      width: 270,
      height: 270,
      tileSize: 256,
      minZoom: 2,
      maxZoom: 9,
    };
    const coordinate = [270, 270];
    const point = { x: 540.0072763538013, y: 539.9927236461986 };

    beforeAll(() => {
      mockedAxiosOldClient
        .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
        .reply(HttpStatusCode.Ok, [ELEMENT_SEARCH_RESULT_MOCK_REACTION]);
    });

    it('returns valid array of objects', async () => {
      const result = await getSearchResults({
        coordinate,
        mapSize,
        modelId,
      });

      expect(result).toEqual({
        point,
        searchResults: [ELEMENT_SEARCH_RESULT_MOCK_REACTION],
      });
    });
  });

  describe('when results type is invalid', () => {
    const { modelId } = ELEMENT_SEARCH_RESULT_MOCK_ALIAS;
    const mapSize = {
      width: 270,
      height: 270,
      tileSize: 256,
      minZoom: 2,
      maxZoom: 9,
    };
    const coordinate = [270, 270];
    const point = { x: 540.0072763538013, y: 539.9927236461986 };

    beforeAll(() => {
      mockedAxiosOldClient
        .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
        .reply(HttpStatusCode.Ok, {
          invalidObject: true,
        });
    });

    it('should return undefined', async () => {
      const result = await getSearchResults({
        coordinate,
        mapSize,
        modelId,
      });

      expect(result).toEqual({
        point,
        searchResults: undefined,
      });
    });
  });
});
