import { DEFAULT_ZOOM } from '@/constants/map';
import { MAP_DATA_INITIAL_STATE } from '@/redux/map/map.constants';
import { GetUpdatedMapDataResult, MapData } from '@/redux/map/map.types';
import { MapBackground, MapModel } from '@/types/models';
import { DeepPartial } from '@reduxjs/toolkit';
import { getPointMerged } from '../object/getPointMerged';

interface GetUpdatedMapDataArgs {
  model: MapModel;
  position?: DeepPartial<MapData['position']>;
  background?: MapBackground;
}

const HALF = 2;

export const getUpdatedMapData = ({
  model,
  position,
  background,
}: GetUpdatedMapDataArgs): GetUpdatedMapDataResult => {
  const defaultPosition = {
    x: model.defaultCenterX ?? model.width / HALF,
    y: model.defaultCenterY ?? model.height / HALF,
    z: model.defaultZoomLevel ?? DEFAULT_ZOOM,
  };

  const mergedPosition = getPointMerged(position?.initial || {}, defaultPosition);

  return {
    modelId: model.idObject,
    backgroundId: background?.id || MAP_DATA_INITIAL_STATE.backgroundId,
    size: {
      width: model.width,
      height: model.height,
      tileSize: model.tileSize,
      minZoom: model.minZoom,
      maxZoom: model.maxZoom,
    },
    position: {
      initial: mergedPosition,
      last: mergedPosition,
    },
  };
};
